///////////////////////////////////////////////////////////////////
//Author: Bo Xiao
//Copyright: Eaglet workroom 2008
//Blog: http://eaglet.cnblogs.com
///////////////////////////////////////////////////////////////////

using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Data.SqlClient;
using System.Diagnostics;

namespace General.Data
{
    /// <summary>
    /// Data reader class for sqlserver
    /// </summary>
    public class SQLDataReader : IDisposable, IDataReader
    {
        SqlConnection _Conn;
        SqlDataReader _DataReader;
        DataTable _SchemaTable;
        DataSet _SchemaDataSet;
        SqlCommand _Command;
        Stopwatch _Stopwatch = new Stopwatch();

        /// <summary>
        /// Get Schema Elapse Milliseconds
        /// </summary>
        public double GetSchemaElapsedMilliseconds
        {
            get
            {
                return _Stopwatch.ElapsedMilliseconds;
            }
        }

        ~SQLDataReader()
        {
            Close();
        }

        #region IDataReader Members

        /// <summary>
        /// Connect dataBase
        /// </summary>
        /// <param name="connectString">Connect string</param>
        public void Connect(string connectString)
        {
            Close();
            _Conn = new SqlConnection(connectString);
            _Conn.Open();
        }

        /// <summary>
        /// Close database
        /// </summary>
        public void Close()
        {
            Dispose();
        }

        /// <summary>
        /// Get Schema
        /// </summary>
        /// <param name="queryString">Query string</param>
        /// <returns>Schema</returns>
        public DataSet GetSchema(string queryString)
        {
            _Stopwatch.Reset();
            _Stopwatch.Start();
            DataSet ds = new DataSet();
            ds.EnforceConstraints = false;

            SqlDataAdapter adapter = new SqlDataAdapter();
            adapter.SelectCommand = new SqlCommand(
                queryString, _Conn);
            adapter.FillSchema(ds, SchemaType.Mapped);
            _Stopwatch.Stop();
            return ds;
        }

        /// <summary>
        /// Open data reader for a query
        /// </summary>
        /// <param name="queryString"></param>
        public void OpenDataReader(string queryString)
        {
            _Command = new SqlCommand(queryString, _Conn);
            _SchemaDataSet = GetSchema(queryString);
            _SchemaTable = _SchemaDataSet.Tables[0];
            _DataReader = _Command.ExecuteReader();
        }

        /// <summary>
        /// Close data reader for a query
        /// </summary>
        public void CloseDataReader()
        {
            try
            {
                if (!_DataReader.IsClosed)
                {
                    _Command.Cancel();
                }
            }
            catch
            {
            }

            _DataReader.Close();
        }

        /// <summary>
        /// Next row
        /// </summary>
        /// <returns>
        /// if reach the end of records, return null, 
        /// else return Datarow
        /// </returns>
        public DataRow NextRow()
        {
            if (_DataReader.Read())
            {
                DataRow row = _SchemaTable.NewRow();
                foreach (DataColumn col in _SchemaTable.Columns)
                {
                    row[col.ColumnName] = _DataReader[col.ColumnName];
                }

                return row;
            }
            else
            {
                return null;
            }
        }


        public DataSet Query(string queryString)
        {
            return RangeQuery(queryString, 0, -1);
        }

        public DataSet RangeQuery(string queryString, long first, long last)
        {
            try
            {
                OpenDataReader(queryString);

                if (first < 0)
                {
                    first = 0;
                }

                for (long i = 0; i < first; i++)
                {
                    if (!_DataReader.Read())
                    {
                        return _SchemaDataSet;
                    }
                }

                if (last < 0)
                {
                    last = 0x7FFFFFFFFFFFFFFF;
                }

                for (long i = first; i <= last; i++)
                {
                    DataRow row = NextRow();

                    if (row != null)
                    {
                        _SchemaTable.Rows.Add(row);
                    }
                    else
                    {
                        return _SchemaDataSet;
                    }
                }

                return _SchemaDataSet;
            }
            finally
            {
                CloseDataReader();
            }
        }

        #endregion

        #region IDisposable Members

        public void Dispose()
        {
            try
            {
                if (_Conn != null)
                {
                    if (_Conn.State != ConnectionState.Closed &&
                        _Conn.State != ConnectionState.Broken)
                    {
                        _Conn.Close();
                    }

                    _Conn = null;
                }
            }
            catch
            {
            }
        }

        #endregion
    }
}
